/**
 * Proyecto: 9x01-Torres de Hanoi
 * Fichero:  Tablero.java
 * Fecha:    25/04/2012
 */
package torresDeHanoi;

public class Tablero
{


	// PROPIEDADES
	private int	  numDiscos;	   // Numero de discos en el Tablero.
	private Torre	tablaTorres[];	// Tabla para las Torres.


	// CONSTRUCTOR
	/**
	 *	<p>El constructor inicializa el tablero con N discos. Si el numero de discos es negativo o cero lo establece a 3 por defecto.</p>
	 *
	 *	@param numDiscos <i>Numero de discos que tendra el Tablero.</i>
	 */
	public Tablero( int numDiscos )
	{
		if( numDiscos <= 0 )
			this.numDiscos = 3;
		else
			this.numDiscos = numDiscos;

		tablaTorres = new Torre[3];
		tablaTorres[0] = new Torre(this.numDiscos, this.numDiscos);
		tablaTorres[1] = new Torre(this.numDiscos, 0);
		tablaTorres[2] = new Torre(this.numDiscos, 0);
	};


	// METODOS
	/**
	 * <p>Indica el numero de movimientos necesarios para N Discos.</p>
	 * @return <i>Devuelve el numero de movimeintos</i>
	 */
	public int getNumMovimientos()
	{
		return (int) Math.pow(2, numDiscos) - 1;
	};


	/**
	 * <p>Muestra los movimientos<br>
	 * -- Primero imprime el tablero<br>
	 * -- Despues llama a hanoiRecursivo</p>
	 */
	public void mostrarLosMovimientos()
	{
		int numDisGrande = tablaTorres[0].getTamanioDisco(0);

		imprimirTablero(numDisGrande);
		hanoiRecursivo( numDiscos, numDisGrande, 0, 1, 2);// NOTA: IMPRIMIR PARECE QUE FUNCIONA... REVISAR HANOIRECURSIVO
	};


	/**
	 * <p>Mediante recursividad, se realizan los N movimientos trasladando los Discos de una Torre a otra.<br>Despues de cada movimiento se imprime el tablero.</p>
	 * 
	 * @param disc				<i>El numero de discos. En cada llamada al procedimiento disminuye en 1.</i>
	 * @param numDisGrande	<i>El numero que representa el Disco mas grande.</i>
	 * @param o					<i>Entero con valor de 0, 1 o 3, que indica el origen del Disco a quitar. En cada iteracion el valor cambia.</i>
	 * @param a					<i>Igual que 'o' pero para marcar la Torre auxiliar.</i>
	 * @param d					<i>Igual que 'o' pero para marcar la Torre destino.</i>
	 */
	private void hanoiRecursivo( int disc, int numDisGrande, int o, int a, int d )
	{
		if( disc == 1 )
		{
			tablaTorres[d].ponerDisco(tablaTorres[o].quitarDisco());
			imprimirTablero(numDisGrande);
		}
		else
		{
			hanoiRecursivo(disc - 1, numDisGrande, o, d, a);

			tablaTorres[d].ponerDisco(tablaTorres[o].quitarDisco());
			imprimirTablero(numDisGrande);

			hanoiRecursivo(disc - 1, numDisGrande, a, o, d);
		};

	}


	/**
	 * <p>Imprime el Tablero por pantalla mediante el terminal de texto.</p>
	 * 
	 * @param tablaTorres	<i>Tablero con 3 Torres.</i>
	 * @param numDisGrande	<i>El numero que representa el Disco mas grande.</i>
	 */
	private void imprimirTablero( int numDisGrande )
	{
		// Recorre las "filas":
		for( int d = numDiscos - 1; d >= 0; d-- )
		{
			// Recorre las Torres:
			for( int t = 0; t < 3; t++ )
			{
				int tam = tablaTorres[t].getTamanioDisco(d);
				int esp = (numDisGrande - tam) / 2;

				// Espacios a la izquierda
				for( int i = 0; i < esp; i++ )
					System.out.print(" ");

				// Imprime los comodines
				for( int i = 0; i < tam; i++ )
					System.out.print("*");

				// Espacios a la derecha
				for( int i = 0; i < esp; i++ )
					System.out.print(" ");

				System.out.print("\t");
			};

			System.out.print("\n");
		};

		// Se pausa el programa:
		try
		{
			Thread.sleep(400);
		}
		catch( InterruptedException ex )
		{
			System.out.println("Delay fail");
		}

		// Se "borra" la pantalla:
		System.out.print("\n\n");

	};

};
